# -*- coding: utf-8 -*-
"""
the checker runner
"""
import trace
from itertools import islice

from specs import parse_method_props
from datagen import DataGenerator


def test_arg_list(prop, m, arg_list, tracer):
    """
    running and testing an argument list
    :param prop: property
    :param m: method
    :param arg_list: argument list
    :param tracer: tracer to track result
    :return: (pre-condition satisfied, post-condition satisfied)
    """
    pre_condition = prop.pre
    post_condition = prop.post
    if pre_condition(*arg_list):
        result = tracer.runfunc(m, *arg_list)
        result_list = (result,) + arg_list
        if post_condition(*result_list):
            return True, True
        else:
            return True, False
    else:
        return False, None


def test_method_prop(method_prop, tracer):
    """
    running and testing a method's properties
    :param method_prop: all properties of a method
    :param tracer: tracer to track result
    :return: (property ast, report)
    """
    m = method_prop.method
    prop = method_prop.prop
    report = []
    for index, arg_list in enumerate(islice(DataGenerator(method_prop), 100)):
        hit, truth = test_arg_list(prop, m, arg_list, tracer)
        print "[{}] running with {}, {}/{}".format(index + 1,
                                                   arg_list,
                                                   "hit" if hit else "miss",
                                                   "true" if truth else "falsified")
        report.append((hit, truth, arg_list))
    return prop.ast, report


def main(filename):
    """
    main runner
    :param filename: filename to open
    :return: nothing
    """
    with open(filename) as f:
        for method_prop in parse_method_props(f.read()):
            tracer = trace.Trace(count=1,
                                 trace=0,
                                 ignoredirs=['sys', 'os'])
            desc, report = test_method_prop(method_prop, tracer)
            hits = filter(lambda x: x[0] is True, report)
            misses = filter(lambda x: x[0] is False, report)
            print desc, \
                'hit:', len(hits), \
                'truth:', len(filter(lambda x: x[1], hits)), \
                'falsified:', len(filter(lambda x: not x[1], hits)), \
                'misses:', len(misses)
            reports = tracer.results()
            reports.write_results(show_missing=True, coverdir=".")


if __name__ == '__main__':
    import sys

    main(sys.argv[1])