# -*- coding: utf-8 -*-
"""
parsing the specs from json property file
"""

from collections import namedtuple
import json
import importlib

from datagen import SUPPORTED_TYPES


Arg = namedtuple('Arg', ['name', 'type'])

Prop = namedtuple('Prop', ['ast', 'pre', 'post'])


class MethodProps(namedtuple('MethodProps', ['method', 'args', 'prop'])):
    def arg_types(self):
        return tuple(arg.type for arg in self.args)


def _parse_expr(string):
    return eval(string)


def _parse_pre(string):
    pre = _parse_expr(string)
    assert callable(pre), 'pre-condition must be callable (lambda)'
    return pre


def _parse_post(string):
    post = _parse_expr(string)
    assert callable(post), 'post-condition must be callable (lambda)'
    return post


def _parse_prop(dictionary):
    return Prop(ast=dictionary,
                pre=_parse_pre(dictionary['pre']),
                post=_parse_post(dictionary['post']))


def _parse_method_name(module_str, method_str):
    module = importlib.import_module(module_str)
    m = getattr(module, method_str)
    assert callable(m), 'must be callable'
    return m


def _parse_arg_type(type_str):
    tp = eval(type_str)
    assert tp in SUPPORTED_TYPES, 'type {} is not supported'.format(tp)
    return tp


def _parse_arg(dictionary):
    return Arg(name=dictionary['name'], type=_parse_arg_type(dictionary['type']))


def parse_method_props(string):
    result = []
    for data in json.loads(string):
        method_props = MethodProps(method=_parse_method_name(data['module'], data['func']),
                                   args=map(_parse_arg, data['args']),
                                   prop=_parse_prop(data['prop']))
        result.append(method_props)
    return result